package Src
{
  import mx.core.*;
  import mx.collections.*;
  import mx.containers.*;
  import flash.display.*;
  import flash.geom.*;
  import flash.events.*;
  import flash.text.*;
  import flash.utils.*;
  import flash.ui.Keyboard;
  import Src.FE.*;
  import Src.Entity.*;
  import Src.Gfx.*;
  import Src.Sound.*;
  import Src.Tiles.*;

  public class Game
  {
    public var T_PHYS:Number=16.6667;
    private var IS_FINAL:Boolean = false;

    public static var STATE_GAME:int = 0;
    public static var STATE_EDITING:int = 1;
    public static var STATE_FE:int = 2;

    public var ui:UIComponent;
    public var canvas:Canvas;

    private var fps:Number=60;
    private var lastTime:int = 0;
    private var fpsText:TextField;

    private var updateTracker:Number = 0;

    private var gameState:int = STATE_GAME;

    public var entityManager:EntityManager;
    public var input:Input;
    public var renderer:Renderer;
    public var soundManager:SoundManager;
    public var tileMap:TileMap;
    public var tileEditor:TileEditor;
    public var frontEnd:Frontend;
    public var camera:Camera;

    public function Game()
    {
      entityManager = new EntityManager(this, 8);
      input = new Input();
      renderer = new Renderer();
      soundManager = new SoundManager();
      tileMap = new TileMap(this);      
      frontEnd = new Frontend(this);
      camera = new Camera(this);
    }

    public function creationComplete():void
    {
      renderer.init();
      soundManager.init();
      tileEditor = new TileEditor(tileMap);

      gameState = STATE_GAME;
      frontEnd.addScreen(new MainMenu());

      fpsText = new TextField();
      fpsText.textColor = 0xffffffff;
      fpsText.text = "352 fps";

      resetEntities();
    }

    private function update():void
    {
      camera.update();
      renderer.update();
      entityManager.update();
      if(gameState == STATE_FE)
        frontEnd.update();
      if(gameState == STATE_EDITING)
        tileEditor.update();
        
      if(input.keyPressedDictionary[69]) // e
      {
        if(gameState == STATE_GAME)
          changeState(STATE_EDITING);
        else
          changeState(STATE_GAME);
        resetEntities();
      }
        
      // Update input last, so mouse presses etc. will register first..
      // also note this mode of operation isn't perfect, sometimes input
      // will be lost!        
      input.update(); 
    }
    
    private function resetEntities():void
    {
      entityManager.reset();      
      if(gameState == STATE_GAME)
        tileMap.spawnEntities();
    }

    private function render():void
    {
      renderer.cls();
      renderer.setCamera(camera);
      tileMap.render();
      entityManager.render();
      renderer.setCamera();
      if(gameState == STATE_EDITING)
        tileEditor.render();      
      if(gameState == STATE_FE)
        frontEnd.render();
      renderer.drawFontText("Jonathan Whiting's Basecode",
                            Renderer.SCR_WIDTH/2, 10, true);

      if(!IS_FINAL)
        renderer.backBuffer.draw(fpsText);
      canvas.graphics.clear();
      renderer.flip(canvas.graphics);
    }

    public function enterFrame(event:Event):void
    {
      var thisTime:int = getTimer();
      fps = (fps*9 + 1000/(thisTime-lastTime))/10;
      updateTracker += thisTime-lastTime;
      lastTime = thisTime;
      if(fpsText)
        fpsText.text = "FPS: "+int(fps);

      while(updateTracker > 0)
      {
        update();
        updateTracker -= T_PHYS;
      }

      if(renderer)
      {
        render();
      }
    }

    public function getState():int
    {
      return gameState;
    }

    public function changeState(state:int):void
    {
      gameState = state;
    }
  }
}